@@ -7,6 +7,14 @@
 #' @param url full url to site
 #' @param cookies DEPRECATED
 #' @export
+#' @note
+#' Because of the way argument dispatch works in R, using handle() in the
+#' http methods (See \code{\link{GET}}) will cause problems when trying to
+#' pass configuration arguments (See examples below). Directly specifying the
+#' handle when using http methods is not recommended in general, since the
+#' selection of the correct handle is taken care of when the user passes an url
+#' (See \code{\link{handle_pool}}).
+#'
 #' @examples
 #' handle("http://google.com")
 #' handle("https://google.com")
@@ -18,6 +26,15 @@
 #'
 #' h <- handle("http://google.com", cookies = FALSE)
 #' GET(handle = h)$cookies
+#'
+#' \dontrun{
+#' # Using the preferred way of configuring the http methods
+#' # will not work when using handle():
+#' GET(handle = h, timeout(10))
+#' # Passing named arguments will work properly:
+#' GET(handle = h, config = list(timeout(10), add_headers(Accept = "")))
+#' }
+#'
 handle <- function(url, cookies = TRUE) {
   stopifnot(is.character(url), length(url) == 1)
 
@@ -34,7 +51,7 @@ print.handle <- function(x, ...) {
 }
 
 ref <- function(x) {
-  str_extract(capture.output(print(x))[1], "0x[0-9a-f]*")
+  str_extract(utils::capture.output(print(x))[1], "0x[0-9a-f]*")
 }
 
 is.handle <- function(x) inherits(x, "handle")
