@@ -36,7 +36,7 @@ progress_bar <- function(type) {
     if (total == 0 && now == 0) {
       bar <<- NULL
       first <<- TRUE
-      return()
+      return(TRUE)
     }
 
     if (total == 0) {
@@ -45,15 +45,15 @@ progress_bar <- function(type) {
       }
       cat("\rDownloading: ", bytes(now, digits = 2), "     ", sep = "")
       if (now == total) cat("\n")
-      flush.console()
+      utils::flush.console()
     } else {
       if (is.null(bar)) {
-        bar <<- txtProgressBar(max = total, style = 3)
+        bar <<- utils::txtProgressBar(max = total, style = 3)
       }
-      setTxtProgressBar(bar, now)
+      utils::setTxtProgressBar(bar, now)
     }
 
-    0L
+    TRUE
   }
 
   show_progress
