@@ -44,20 +44,21 @@ status_code.numeric <- function(x) x
 http_status <- function(x) {
   status <- status_code(x)
 
-  status_desc <- http_statuses[as.character(status)]
+  status_desc <- http_statuses[[as.character(status)]]
   if (is.na(status_desc)) {
     stop("Unknown http status code: ", status, call. = FALSE)
   }
 
-  status_types <- c("information", "success", "redirection", "client error",
-    "server error")
-  status_type <- status_types[status %/% 100]
+  status_types <- c("Information", "Success", "Redirection", "Client error",
+    "Server error")
+  status_type <- status_types[[status %/% 100]]
 
   # create the final information message
   message <- paste(status_type, ": (", status, ") ", status_desc, sep = "")
 
   list(
     category = status_type,
+    reason = status_desc,
     message = message
   )
 }
@@ -135,46 +136,67 @@ http_statuses <- c(
   "599" = "Network connect timeout error (Unknown)"
 )
 
-#' Check for an http OK status.
+#' Check for an http error.
 #'
-#' Checks if a request on a given URL succeeds and returns an OK status.
-#'
-#' @param ... passed to HEAD to perform the request (usually just a url)
-#' @return This function returns \code{TRUE} only if the http status is
-#'   exactly 200, \code{FALSE otherwise}. See
-#'   \code{http://en.wikipedia.org/wiki/Http_status_codes}
-#'   for more information on http status codes.
+#' @param x Object to check. Default methods are provided for strings
+#'   (which perform an \code{\link{HEAD}} request), responses, and
+#'   integer status codes.
+#' @param ... Other arguments passed on to methods.
+#' @return  \code{TRUE} if the request fails (status code 400 or above),
+#'   otherwise \code{FALSE}.
 #' @export
 #' @family response methods
 #' @examples
-#' url_ok("http://www.google.com")
-#' url_ok("http://httpbin.org/status/200")
-#' url_ok("http://httpbin.org/status/201")
-url_ok <- function(...) {
-  x <- HEAD(...)
-  identical(status_code(x), 200L)
+#' # You can pass a url:
+#' http_error("http://www.google.com")
+#' http_error("http://httpbin.org/status/404")
+#'
+#' # Or a request
+#' r <- GET("http://httpbin.org/status/201")
+#' http_error(r)
+#'
+#' # Or an (integer) status code
+#' http_error(200L)
+#' http_error(404L)
+http_error <- function(x, ...) {
+  UseMethod("http_error")
+}
+#' @export
+http_error.character <- function(x, ...) {
+  http_error(HEAD(x, ...))
+}
+#' @export
+http_error.response <- function(x, ...) {
+  http_error(status_code(x))
 }
 
-#' Check for an http success status.
-#'
-#' Checks if a request on a given URL succeeds.
-#'
-#' @param ... passed to HEAD to perform the request (usually just a url)
-#' @return This function returns \code{TRUE} if the request succeeds
-#'   (status in the 200s), \code{FALSE otherwise}. See
-#'   \code{http://en.wikipedia.org/wiki/Http_status_codes}
-#'   for more information on http status codes.
 #' @export
-#' @family response methods
-#' @examples
-#' url_success("http://www.google.com")
-#' url_success("http://httpbin.org/status/200")
-#' url_success("http://httpbin.org/status/201")
-#' url_success("http://httpbin.org/status/300")
-url_success <- function(...) {
-  successful(HEAD(...))
+http_error.integer <- function(x, ...) {
+  x >= 400L
+}
+
+#' @export
+#' @rdname http_error
+#' @usage NULL
+url_success <- function(x, ...) {
+  warning(
+    "`url_success(x)` is deprecated; please use `!http_error(x)` instead.",
+    call. = FALSE
+  )
+  !http_error(x, ...)
+}
+
+#' @export
+#' @rdname http_error
+#' @usage NULL
+url_ok <- function(x, ...) {
+  warning(
+    "`url_ok(x)` is deprecated; ",
+    "please use `identical(status_code(x), 200L)` instead.",
+    call. = FALSE
+  )
+  identical(status_code(HEAD(x, ...)), 200L)
 }
-successful <- function(x) status_code(x) < 300
 
 #' Take action on http error.
 #'
@@ -182,43 +204,61 @@ successful <- function(x) status_code(x) < 300
 #' be used whenever you're creating requests inside a function, so
 #' that the user knows why a request has failed.
 #'
-#' @return If request was successful, an invisible \code{TRUE}. Otherwise,
+#' @return If request was successful, the response (invisibly. Otherwise,
 #'   raised a classed http error or warning, as generated by
 #'   \code{\link{http_condition}}
+#' @inheritParams http_condition
 #' @seealso \code{\link{http_status}} and
 #'   \code{http://en.wikipedia.org/wiki/Http_status_codes} for more information
 #'   on http status codes.
-#' @param x a request object
 #' @family response methods
 #' @examples
 #' x <- GET("http://httpbin.org/status/200")
 #' stop_for_status(x) # nothing happens
 #' warn_for_status(x)
+#' message_for_status(x)
 #'
 #' x <- GET("http://httpbin.org/status/300")
 #' \dontrun{stop_for_status(x)}
 #' warn_for_status(x)
+#' message_for_status(x)
 #'
 #' x <- GET("http://httpbin.org/status/404")
 #' \dontrun{stop_for_status(x)}
 #' warn_for_status(x)
-#' @name http_error
-NULL
-
-#' @rdname http_error
+#' message_for_status(x)
+#'
+#' # You can provide more information with the task argumgnet
+#' warn_for_status(x, "download spreadsheet")
+#' message_for_status(x, "download spreadsheet")
+#'
 #' @export
-stop_for_status <- function(x) {
-  if (successful(x)) return(invisible(TRUE))
-  stop(http_condition(x, "error", call = sys.call(-1)))
+stop_for_status <- function(x, task = NULL) {
+  if (status_code(x) < 300)
+    return(invisible(x))
+
+  call <- sys.call(-1)
+  stop(http_condition(x, "error", task = task, call = call))
 }
 
-#' @rdname http_error
+#' @rdname stop_for_status
 #' @export
-warn_for_status <- function(x) {
-  if (successful(x)) return(invisible(TRUE))
-  warning(http_condition(x, "warning", call = sys.call(-1)))
+warn_for_status <- function(x, task = NULL) {
+  if (status_code(x) < 300)
+    return(invisible(x))
+
+  call <- sys.call(-1)
+  warning(http_condition(x, "warning", task = task, call = call))
 }
 
+#' @rdname stop_for_status
+#' @export
+message_for_status <- function(x, task = NULL) {
+  call <- sys.call(-1)
+  message(http_condition(x, "message", task = task, call = call))
+}
+
+
 #' Generate a classed http condition.
 #'
 #' This function generate S3 condition objects which are passed to
@@ -233,8 +273,9 @@ warn_for_status <- function(x) {
 #'   \code{status_code} method)
 #' @param type type of condition to generate. Must be one of error,
 #'   warning or message.
-#' @param message The text of the message. If \code{NULL}, the default,
-#'   will use \code{\link{http_status}} to generate the message.
+#' @param task The text of the message: either \code{NULL} or a
+#'   character vector. If non-\code{NULL}, the error message will finish with
+#'   "Failed to \code{task}".
 #' @param call The call stored in the condition object.
 #' @seealso
 #'   \url{http://adv-r.had.co.nz/Exceptions-Debugging.html#condition-handling}
@@ -256,15 +297,24 @@ warn_for_status <- function(x) {
 #' f("http://httpbin.org/status/404")
 #' f("http://httpbin.org/status/403")
 #' f("http://httpbin.org/status/505")
-http_condition <- function(x, type, message = NULL, call = sys.call(-1)) {
-  status <- status_code(x)
-  status_type <- (status %/% 100) * 100
-
+http_condition <- function(x, type, task = NULL, call = sys.call(-1)) {
   type <- match.arg(type, c("error", "warning", "message"))
 
-  message <- message %||% http_status(status)$message
+  if (is.null(task)) {
+    task <- ""
+  } else if (is.character(task)) {
+    task <- paste0(" Failed to ", task, ".")
+  } else {
+    stop("`task` must be NULL or a character vector", call. = FALSE)
+  }
 
-  http_class <- paste("http_", c(status, status_type, "error"), sep = "")
+  status <- status_code(x)
+  reason <- http_status(status)$reason
+
+  message <- sprintf("%s (HTTP %d).%s", reason, status, task)
+
+  status_type <- (status %/% 100) * 100
+  http_class <- paste0("http_", unique(c(status, status_type, "error")))
 
   structure(
     list(message = message, call = call),
