@@ -116,3 +116,7 @@ github_pat <- function(force = FALSE) {
   pat
 }
 
+## ------------------------------------------------------------------------
+NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
+knitr::opts_chunk$set(purl = NOT_CRAN)
+
